# -*- coding: utf-8 -*-

# --------------------------------------------------------------------------
# Copyright Commvault Systems, Inc.
# See LICENSE.txt in the project root for
# license information.
# --------------------------------------------------------------------------

""""Main file for executing this test case

TestCase is the only class defined in this file.

TestCase: Class for executing this test case

TestCase:
    __init__()      --  initialize TestCase class

    run()           --  run function of this test case
"""
import os
import time

from AutomationUtils.cvtestcase import CVTestCase
from AutomationUtils.constants import AUTOMATION_DIRECTORY
from Reports.utils import TestCaseUtils
from Web.Common.exceptions import CVWebAutomationException
from Web.AdminConsole.Helper.GDPRHelper import GDPR
from Web.AdminConsole.adminconsole import AdminConsole
from Web.Common.cvbrowser import BrowserFactory, Browser
from Web.Common.exceptions import CVTestCaseInitFailure, CVTestStepFailure
from Web.Common.page_object import handle_testcase_exception, TestStep
from dynamicindex.utils.activateutils import ActivateUtils


class TestCase(CVTestCase):
    """Class for executing Move action in Review page for a file"""
    test_step = TestStep()

    def __init__(self):
        """Initializes test case class object"""
        super(TestCase, self).__init__()
        self.activateutils = ActivateUtils()
        self.testcaseutils = CVTestCase
        self.name = "Basic acceptance test for Move action in Review page in AdminConsole"
        self.tcinputs = {
            "IndexServerName": None,
            "ContentAnalyzer": None,
            "NameServerAsset": None,
            "HostNameToAnalyze": None,
            "FileServerDirectoryPath": None,
            "FileServerUserName": None,
            "FileServerPassword": None,
            "DestinationPath": None,
            "Username": None,
            "Password": None
        }
        # Testcase constants
        self.inventory_name = None
        self.plan_name = None
        self.project_name = None
        self.file_server_display_name = None
        self.country_name = None
        self.browser = None
        self.admin_console = None
        self.utils = TestCaseUtils(self)
        self.navigator = None
        self.test_case_error = None
        self.gdpr_base = None
        self.explict_wait = 1 * 60
        self.file_count_before_review = 0

    def generate_sensitive_data(self):
        """
            Generate sensitive files with PII entities
        """
        self.activateutils.sensitive_data_generation(self.tcinputs['FileServerDirectoryPath'])

    def get_sensitive_file_details(self):
        """
            Get the sensitive file with entity
        """
        entity_delimiter = "****"
        filename = os.path.join(AUTOMATION_DIRECTORY, 'CompiledBins')
        database_file_path = "{0}\\Entity.db".format(filename)
        entities_dict = self.activateutils.db_get_entities_dict_from_sqllite(
            database_file_path)
        dict_used = entities_dict.pop('result')
        self.log.info(dict_used)
        __entity = ""
        __file = ""
        for item in dict_used:
            __file = item.get('FilePath')
            __entity = item.get('Email')
            if __entity is not None:
                if entity_delimiter in __entity:
                    temp = __entity.split(entity_delimiter)
                    __entity = temp[0]
                    break
                else:
                    __entity = __entity
                    break

        self.sensitive_entity = __entity
        self.sensitive_file = __file

    def delete_files_in_folder(self, destination_path):
        for root, dirs, files in os.walk(destination_path):
            for file in files:
                os.remove(os.path.join(root, file))

    def init_tc(self):
        """ Initial configuration for the test case"""
        try:
            self.username = self.inputJSONnode['commcell']['commcellUsername']
            self.password = self.inputJSONnode['commcell']['commcellPassword']
            self.inventory_name = f'{self.id}_inventory'
            self.plan_name = f'{self.id}_plan'
            self.project_name = f'{self.id}_project'
            self.browser = BrowserFactory().create_browser_object()
            self.browser.open()
            self.admin_console = AdminConsole(self.browser, self.commcell.webconsole_hostname)
            self.admin_console.login(username=self.username, password=self.password)
            self.navigator = self.admin_console.navigator
            self.navigator.navigate_to_governance_apps()
            self.gdpr_base = GDPR(self.admin_console, self.commcell, self.csdb)
            self.cleanup()

        except Exception as exception:
            raise CVTestCaseInitFailure(exception)from exception

    @test_step
    def create_inventory(self):
        """
            Create an inventory with a nameserver
        """
        self.navigator.navigate_to_governance_apps()
        self.gdpr_base.inventory_details_obj.select_inventory_manager()
        self.gdpr_base.inventory_details_obj.add_inventory(
            self.inventory_name, self.tcinputs['IndexServerName'])

        self.gdpr_base.inventory_details_obj.add_asset_name_server(
            self.tcinputs["NameServerAsset"])
        self.admin_console.log.info("Sleeping for [%s]", self.explict_wait)
        time.sleep(self.explict_wait)
        if not self.gdpr_base.inventory_details_obj.wait_for_asset_status_completion(
                self.tcinputs['NameServerAsset']):
            raise CVTestStepFailure("Could not complete asset scan.")

    @test_step
    def create_plan(self):
        """
            Creates a plan
        """

        self.navigator.navigate_to_plan()
        self.gdpr_base.plans_obj.create_data_classification_plan(self.plan_name, self.tcinputs[
            'IndexServerName'], self.tcinputs['ContentAnalyzer'],
            None, select_all=True)

    @test_step
    def create_sda_project(self):
        """
            Creates a project and runs analysis
        """
        self.file_server_display_name = '{}_file_server'.format(self.id)
        self.country_name = 'United States'
        self.gdpr_base.testdata_path = self.tcinputs['FileServerDirectoryPath']
        self.gdpr_base.data_source_name = self.file_server_display_name
        self.navigator.navigate_to_governance_apps()
        self.gdpr_base.inventory_details_obj.select_sensitive_data_analysis()
        self.gdpr_base.file_server_lookup_obj.add_project(
            self.project_name, self.plan_name, self.inventory_name)
        self.gdpr_base.file_server_lookup_obj.select_add_data_source()
        self.gdpr_base.file_server_lookup_obj.add_file_server(
            self.tcinputs['HostNameToAnalyze'], 'Client name',
            self.file_server_display_name, self.country_name,
            self.tcinputs['FileServerDirectoryPath'],
            self.tcinputs['FileServerUserName'],
            self.tcinputs['FileServerPassword'])
        self.log.info("Sleeping for: '[%s]' seconds", self.explict_wait)
        time.sleep(self.explict_wait)
        if not self.gdpr_base.file_server_lookup_obj.wait_for_data_source_status_completion(
                self.file_server_display_name, timeout=60):
            raise CVTestStepFailure("Could not complete Data Source scan.")
        self.log.info("Sleeping for: '[%s]' seconds", self.explict_wait)
        time.sleep(self.explict_wait)
        self.file_count_before_review = int(self.gdpr_base.data_source_discover_obj.
                                            get_total_number_after_crawl())

    @test_step
    def review_move_action_filesystem(self):
        """
        Performs move action on a file for a filesystem datasource and verifies that the file is moved to the destination path correctly
        """
        file_count_before_review = 0
        file_count_after_review = 0
        filepath = self.sensitive_file
        filename = filepath[filepath.rindex("\\") + 1:]
        destination_path = self.tcinputs["DestinationPath"]
        username = self.tcinputs["Username"]
        password = self.tcinputs["Password"]
        try:
            self.gdpr_base.data_source_review_obj.review_move_action(
                filename, destination_path, username, password)
            file_count_after_review = sum([len(files)
                                           for r, d, files in os.walk(destination_path)])
            self.log.info(
                "Number of files left after move operation is performed: %d",
                file_count_after_review)
            if file_count_after_review == 1:
                self.log.info("Successfully moved {0} to {1}".format(filename, destination_path))
            else:
                raise CVTestStepFailure
        except (CVWebAutomationException, CVTestStepFailure) as error_status:
            self.test_case_error = f'Move action failed:- {str(error_status)}'
            self.gdpr_base.data_source_review_obj.close_action_modal()

    @test_step
    def review_move_action_full_crawl_filesystem(self):
        """
        Verifies that the moved file is not getting picked during a full crawl
        """
        self.navigator.navigate_to_governance_apps()
        self.gdpr_base.inventory_details_obj.select_sensitive_data_analysis()
        self.gdpr_base.file_server_lookup_obj.search_for_project(self.project_name)
        self.gdpr_base.data_source_discover_obj.navigate_to_project_details(self.project_name)
        self.gdpr_base.file_server_lookup_obj.select_data_source(self.file_server_display_name)
        self.log.info("Starting a full re-crawl of the datasource [%s]",
                      self.file_server_display_name)
        self.gdpr_base.data_source_discover_obj.select_details()
        self.gdpr_base.data_source_discover_obj.start_data_collection_job('full')
        self.navigator.navigate_to_governance_apps()
        self.gdpr_base.inventory_details_obj.select_sensitive_data_analysis()
        self.gdpr_base.file_server_lookup_obj.navigate_to_project_details(
            self.project_name)
        if not self.gdpr_base.file_server_lookup_obj.wait_for_data_source_status_completion(
                self.file_server_display_name):
            raise Exception("Could not complete Datasource scan.")
        self.log.info("Sleeping for: '[%s]' seconds", self.explict_wait)
        time.sleep(self.explict_wait)
        file_count_after_review = int(self.gdpr_base.data_source_discover_obj.
                                      get_total_number_after_crawl())
        if self.file_count_before_review - 1 != file_count_after_review:
            self.log.info(
                "Number of files before performing Move operation [%d], "
                "Number of files after performing Move operation [%d]",
                self.file_count_before_review,
                file_count_after_review)
            self.test_case_error = (
                "Number of files before performing Move operation [%d], number of files after performing Move operation [%d]. "
                "Move operation failed.", self.file_count_before_review, file_count_after_review)
        else:
            self.log.info(
                "Number of files before performing Move operation [%d], "
                "Number of files after performing Move operation [%d]",
                self.file_count_before_review,
                file_count_after_review)

    @test_step
    def cleanup(self):
        """
            Cleans up the environment
        """
        self.navigator.navigate_to_governance_apps()
        self.gdpr_base.cleanup(
            self.project_name,
            self.inventory_name,
            self.plan_name, pseudo_client_name=self.file_server_display_name)
        self.delete_files_in_folder(self.tcinputs["DestinationPath"])

    def run(self):
        try:
            self.init_tc()
            self.generate_sensitive_data()
            self.get_sensitive_file_details()
            self.create_inventory()
            self.create_plan()
            self.create_sda_project()
            self.gdpr_base.file_server_lookup_obj.select_data_source(
                self.file_server_display_name)
            self.gdpr_base.data_source_discover_obj.select_review()
            self.review_move_action_filesystem()
            self.review_move_action_full_crawl_filesystem()
            self.cleanup()
            if self.test_case_error is not None:
                raise Exception(self.test_case_error)
        except Exception as err:
            handle_testcase_exception(self, err)
        finally:
            AdminConsole.logout_silently(self.admin_console)
            Browser.close_silently(self.browser)
